/*
    PPSRGBUtilities.m

    Copyright 2013-2016 Josh Freeman
    http://www.twilightedge.com

    This file is part of PikoPixel for Mac OS X and GNUstep.
    PikoPixel is a graphical application for drawing & editing pixel-art images.

    PikoPixel is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License version 3,
    as published by the Free Software Foundation.

    PikoPixel is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#import "PPSRGBUtilities.h"

#import "PPManagedColor.h"


#define macroLinearComponentForSRGBComponent(sRGBComponent)                             \
            ((sRGBComponent > 0.04045f) ?                                               \
                powf((sRGBComponent + 0.055f) / 1.055f, 2.4f) :                         \
                sRGBComponent / 12.92f)

#define macroSRGBComponentForLinearComponent(linearComponent)                           \
            ((linearComponent > 0.0031308f) ?                                           \
                1.055f * powf(linearComponent, 1.0f / 2.4f) - 0.055f :                  \
                linearComponent * 12.92f)


@implementation NSColor (PPSRGBUtilities)

- (NSColor *) ppSRGBUtils_BlendedSRGBColorWithFraction: (CGFloat) fraction
                ofColor: (NSColor *) otherColor
{
    CGFloat color1Fraction, color2Fraction, color1Components_sRGB[4], color2Components_sRGB[4],
            color1Component_Linear, color2Component_Linear, blendedComponent_Linear,
            blendedComponents_sRGB[4];
    int componentIndex;

    self = [self ppManagedColor_sRGBColor];
    otherColor = [otherColor ppManagedColor_sRGBColor];

    if (!self || !otherColor)
    {
        goto ERROR;
    }

    if (fraction >= 1.0)
    {
        return otherColor;
    }
    else if (fraction <= 0.0)
    {
        return self;
    }

    color1Fraction = 1.0 - fraction;
    color2Fraction = fraction;

    [self getComponents: color1Components_sRGB];
    [otherColor getComponents: color2Components_sRGB];

    // RGB channels

    for (componentIndex=0; componentIndex<3; componentIndex++)
    {
        color1Component_Linear =
            macroLinearComponentForSRGBComponent(color1Components_sRGB[componentIndex]);

        color2Component_Linear =
            macroLinearComponentForSRGBComponent(color2Components_sRGB[componentIndex]);

        blendedComponent_Linear = color1Fraction * color1Component_Linear
                                    + color2Fraction * color2Component_Linear;

        blendedComponents_sRGB[componentIndex] =
            macroSRGBComponentForLinearComponent(blendedComponent_Linear);
    }

    // Alpha channel

    blendedComponents_sRGB[3] = color1Fraction * color1Components_sRGB[3]
                                    + color2Fraction * color2Components_sRGB[3];

    return [NSColor ppManagedColor_sRGBColorWithRed: blendedComponents_sRGB[0]
                                            green: blendedComponents_sRGB[1]
                                            blue: blendedComponents_sRGB[2]
                                            alpha: blendedComponents_sRGB[3]];

ERROR:
    return nil;
}

@end
